#!/bin/bash -e
# topo B-A-C creates the following network topology:
#
#
#     ns:B      ns:A      ns:C
#
#       B  -----  A  -----  C
#        b-a   a-b a-c   c-a
#
#
# if run with -cycle, it also creates B-C link:
#
#
#       B  ...............  C
#       |                   |
#       '-------------------'
#
# should be run under unshare -mrun .
# use xnsenter {A,B,C} to enter into namespaces of A/B/C.
#
# Based on https://github.com/troglobit/smcroute/tree/master/test

. lib.sh

cycle=n
[ "$1" = "-cycle" ] && cycle=y

# set up hosts
xnewhost A 2
xnewhost B
xnewhost C

# links
xlink B A   # B-A
xlink A C   # A-C
[ $cycle = y ] && xlink B C

set -x

# addresses
xnsenter B -- ip addr add 10.0.0.1/24 dev b-a
xnsenter A -- ip addr add 10.0.0.2/24 dev a-b
xnsenter C -- ip addr add 20.0.0.1/24 dev c-a
xnsenter A -- ip addr add 20.0.0.2/24 dev a-c
if [ $cycle = y ]; then
    xnsenter B -- ip addr add 30.0.0.1/24 dev b-c
    xnsenter C -- ip addr add 30.0.0.2/24 dev c-b
fi

# multicast routing
xnsenter B -- ip route add 224.0.0.0/4 dev b-a
xnsenter C -- ip route add 224.0.0.0/4 dev c-a

xnsenter A -- ip route add    224.0.0.0/4 dev a-b   # NOTE on A 224.0.0.0/4 is routed
xnsenter A -- ip route append 224.0.0.0/4 dev a-c   # to _both_ a-b and a-c

if [ $cycle = y ]; then
    xnsenter B -- ip route append 224.0.0.0/4 dev b-c
    xnsenter C -- ip route append 224.0.0.0/4 dev c-b
fi
