#!/bin/bash -ex
# topo square creates the following network topology:
#
#
#        A --------- br0 -------- B
#        |  ab                ba  |
#        |ac                    bd|
#        |                        |
#        |                        |
#        |                        |
#       br1                      br3
#        |                        |
#        |                        |
#        |                        |
#        |ca                    db|
#        |   cd               dc  |
#        C ---------- br2 ------- D
#
#
#
# should be run under unshare -mrun .
# use xnsenter {A,B,C,D} to enter into namespaces of A/B/C/D.
#
# Based on https://github.com/troglobit/smcroute/tree/master/test

. lib.sh

PID=$$

# A-br0-B
xunshare B -- ip link set lo up
xnsenter B -- ip link add ba type veth peer BA
xnsenter B -- ip link set BA netns $PID
xnsenter B -- ip link set ba up
ip link set BA up

xunshare A -- ip link set lo up
xnsenter A -- ip link add ab type veth peer AB
xnsenter A -- ip link set AB netns $PID
xnsenter A -- ip link set ab up
ip link set AB up

ip link add br0 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set BA master br0
ip link set AB master br0
ip link set br0 up

# A-br1-C
xnsenter A -- ip link add ac type veth peer AC
xnsenter A -- ip link set AC netns $PID
xnsenter A -- ip link set ac up
ip link set AC up

xunshare C -- ip link set lo up
xnsenter C -- ip link add ca type veth peer CA
xnsenter C -- ip link set CA netns $PID
xnsenter C -- ip link set ca up
ip link set CA up

ip link add br1 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set CA master br1
ip link set AC master br1
ip link set br1 up

# C-br2-D
xnsenter C -- ip link add cd type veth peer CD
xnsenter C -- ip link set CD netns $PID
xnsenter C -- ip link set cd up
ip link set CD up

xunshare D -- ip link set lo up
xnsenter D -- ip link add dc type veth peer DC
xnsenter D -- ip link set DC netns $PID
xnsenter D -- ip link set dc up
ip link set DC up

ip link add br2 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set CD master br2
ip link set DC master br2
ip link set br2 up

# B-br3-D
xnsenter B -- ip link add bd type veth peer BD
xnsenter B -- ip link set BD netns $PID
xnsenter B -- ip link set bd up
ip link set BD up

xnsenter D -- ip link add db type veth peer DB
xnsenter D -- ip link set DB netns $PID
xnsenter D -- ip link set db up
ip link set DB up

ip link add br3 type bridge # vlan_filtering 1 mcast_snooping 0
ip link set BD master br3
ip link set DB master br3
ip link set br3 up

# addresses
xnsenter B -- ip addr add 10.0.0.1/24 dev ba
xnsenter A -- ip addr add 10.0.0.2/24 dev ab
xnsenter C -- ip addr add 20.0.0.1/24 dev ca
xnsenter A -- ip addr add 20.0.0.2/24 dev ac
xnsenter B -- ip addr add 30.0.0.1/24 dev bd
xnsenter D -- ip addr add 30.0.0.2/24 dev db
xnsenter C -- ip addr add 40.0.0.1/24 dev cd
xnsenter D -- ip addr add 40.0.0.2/24 dev dc

# IPv6 addresses
xnsenter A -- ip addr add 2001:db8:1::1/48 dev lo
xnsenter B -- ip addr add 2001:db8:2::1/48 dev lo
xnsenter C -- ip addr add 2001:db8:3::1/48 dev lo
xnsenter D -- ip addr add 2001:db8:4::1/48 dev lo

# we need to mount private /var/run in each namespace so that pim-dm and babel can be started
xnsenter A -- mount -t tmpfs none /var/run
xnsenter B -- mount -t tmpfs none /var/run
xnsenter C -- mount -t tmpfs none /var/run
xnsenter D -- mount -t tmpfs none /var/run
xnsenter A -- mount -t tmpfs none /var/log
xnsenter B -- mount -t tmpfs none /var/log
xnsenter C -- mount -t tmpfs none /var/log
xnsenter D -- mount -t tmpfs none /var/log
sleep 2

# babeld
xnsenter A -- /opt/re6st/parts/babeld/bin/babeld -h 15 -H 15 -L /var/log/babeld.log -S /var/run/babeld.state -I /var/run/babeld.pid -s -C "ipv6-subtrees true" -C "redistribute local deny" -C "redistribute ip 2001:db8:1::1/48 eq 48" -C "default max-rtt-penalty 5000 rtt-max 500 rtt-decay 125" -C "install ip ::/0 eq 0 src-prefix 2001:db8::/32" -C "redistribute deny" -C "install pref-src 2001:db8:1::1" -C "interface ab type tunnel" -C "interface ac type tunnel" ab ac &
xnsenter B -- /opt/re6st/parts/babeld/bin/babeld -h 15 -H 15 -L /var/log/babeld.log -S /var/run/babeld.state -I /var/run/babeld.pid -s -C "ipv6-subtrees true" -C "redistribute local deny" -C "redistribute ip 2001:db8:2::1/48 eq 48" -C "default max-rtt-penalty 5000 rtt-max 500 rtt-decay 125" -C "install ip ::/0 eq 0 src-prefix 2001:db8::/32" -C "redistribute deny" -C "install pref-src 2001:db8:2::1" -C "interface ba type tunnel" -C "interface bd type tunnel" ba bd &
xnsenter C -- /opt/re6st/parts/babeld/bin/babeld -h 15 -H 15 -L /var/log/babeld.log -S /var/run/babeld.state -I /var/run/babeld.pid -s -C "ipv6-subtrees true" -C "redistribute local deny" -C "redistribute ip 2001:db8:3::1/48 eq 48" -C "default max-rtt-penalty 5000 rtt-max 500 rtt-decay 125" -C "install ip ::/0 eq 0 src-prefix 2001:db8::/32" -C "redistribute deny" -C "install pref-src 2001:db8:3::1" -C "interface cd type tunnel" -C "interface ca type tunnel" cd ca &
xnsenter D -- /opt/re6st/parts/babeld/bin/babeld -h 15 -H 15 -L /var/log/babeld.log -S /var/run/babeld.state -I /var/run/babeld.pid -s -C "ipv6-subtrees true" -C "redistribute local deny" -C "redistribute ip 2001:db8:4::1/48 eq 48" -C "default max-rtt-penalty 5000 rtt-max 500 rtt-decay 125" -C "install ip ::/0 eq 0 src-prefix 2001:db8::/32" -C "redistribute deny" -C "install pref-src 2001:db8:4::1" -C "interface db type tunnel" -C "interface dc type tunnel" db dc &

# pim-dm
sed -e 's/IF0/ab/g' -e 's/IF1/ac/g' pim-dm.yaml.tmpl > pim-dm_A.yaml
sed -e 's/IF0/ba/g' -e 's/IF1/bd/g' pim-dm.yaml.tmpl > pim-dm_B.yaml
sed -e 's/IF0/ca/g' -e 's/IF1/cd/g' pim-dm.yaml.tmpl > pim-dm_C.yaml
sed -e 's/IF0/dc/g' -e 's/IF1/db/g' pim-dm.yaml.tmpl > pim-dm_D.yaml
xnsenter A -- pim-dm -config pim-dm_A.yaml &
xnsenter B -- pim-dm -config pim-dm_B.yaml &
xnsenter C -- pim-dm -config pim-dm_C.yaml &
xnsenter D -- pim-dm -config pim-dm_D.yaml &

# open terminals in A B C D for play
xnsenter A -- xfce4-terminal --title=A &
xnsenter B -- xfce4-terminal --title=B &
xnsenter C -- xfce4-terminal --title=C &
xnsenter D -- xfce4-terminal --title=D &
